// new definition for a list that contains all players (max 4)
PLAYERCONTAINER* initPlayerList() {
	PLAYERCONTAINER* tempList = sys_malloc(sizeof(PLAYERCONTAINER));
	
	tempList.playerStoreNumber = 0;
	
	var i;
	for(i=0; i<4; i++) {
		tempList.playerArray[i] = NULL;
		tempList.lifes[i] = 0;
	}
	
	return(tempList);
}

void storeNewPlayer(PLAYER* newPlayer) {
	// check if available to add new player
	if(gPlayerList.playerStoreNumber >= 4) {
		error("There are more then 4 players, this is unsupported!");
		return;
	}
	
	// save the player in the list and increase playercount
	if(gPlayerList.playerArray[newPlayer.playerID] == NULL) { // if the player was not yet identified previously
		gPlayerList.playerStoreNumber += 1;
		gPlayerList.lifes[newPlayer.playerID] = 3; // give 3 lifes to every player
	}
	gPlayerList.playerArray[newPlayer.playerID] = newPlayer;
}

// new definition of previously created struct
PLAYER* initmyPlayer() {
	// initialise memory for new item
	PLAYER* myPlayer = sys_nxalloc( sizeof(PLAYER) );
	
	// setup all variables
	myPlayer.myChar.Character = my;
	myPlayer.myChar.movementMode = moving;
	myPlayer.myChar.animationMode = moving;
	myPlayer.myChar.gravityOffset = my.skill1;
	myPlayer.myChar.health = my.skill2;
	
	myPlayer.pressedMoveKey = 0;
	
	myPlayer.curHeading = 0;
	vec_set(myPlayer.playerRot, nullvector);
	
	// animation variables
	myPlayer.myChar.anim.currentblend = 0;
	myPlayer.myChar.anim.animate = 0;
	myPlayer.myChar.anim.animate2 = 0;
	myPlayer.myChar.anim.animblend = 0;
	myPlayer.myChar.anim.previousframe = 0;
	myPlayer.myChar.anim.currentframe = 0;
	myPlayer.myChar.anim.blendframe = 0;
	
	myPlayer.playerReset = 0;
	myPlayer.playerID = my.skill3;
	
	// get all keys for movement
	getKeyMapping(myPlayer); // return specific player's key seyp from file
	initKeyInputCombo(myPlayer.attackInput); // empty combo string
	myPlayer.comboCount = 0;
	
	// run it's own unique function
	storeNewPlayer(myPlayer); // add player to global list
	RunmyChar(myPlayer); // create player, can be created to use multiple players...
	initAnimation(myPlayer.myChar.anim, myPlayer.myChar.Character, 10); // can be initiated for everyone
	
	// create HUD for individual player
	switch(myPlayer.playerID) {
		case 0:
			addNewPlayerHUD(myPlayer, -128, -128, 128+32, 128-25);
			// player 1 (left top)
			break;
			
		case 1:
			addNewPlayerHUD(myPlayer, screen_size.x - (512-128), -128, 128-32, 128-25);
			// player 2 (right top)
			break;
	}
	
	// other players setup (not implemented in this demo)
//	addNewPlayerHUD(myPlayer, -64, screen_size.y - (256-64));// player 3 (left bottom)
//	addNewPlayerHUD(myPlayer, screen_size.x - (512-64), screen_size.y - (256-64));// player 4 (right bottom)
	
	// return pointer
	return(myPlayer);
}

// get keys input
void getInput(PLAYER* tempChar) {
	VECTOR vecTemp;
	tempChar.pressedMoveKey = 0;
	
	var moveUp = key_pressed(key_for_str(tempChar.keySetup[0]));
	var moveDown = key_pressed(key_for_str(tempChar.keySetup[2]));
	var moveLeft = key_pressed(key_for_str(tempChar.keySetup[1]));
	var moveRight = key_pressed(key_for_str(tempChar.keySetup[3]));
	
	if(moveUp || moveDown || moveLeft || moveRight) { // if any movement key is pressed...
		vec_set(vecTemp, vector(moveUp - moveDown, moveLeft - moveRight, 0)); // store movement
		vec_to_angle(vecTemp, vecTemp);
		
		tempChar.curHeading = vecTemp.x;
		tempChar.pressedMoveKey = 1;
	}
}

// unique function that runs on every struct (main loop for each item)
void RunmyChar(PLAYER* tempChar) {
	VECTOR tempCamRot;
	int prevComboRead = 0;
	int currentComboRead = 0;
	int nextAttackCheck = 0;
	int nextDamageCheck = 0;
	
	int attackTraceDist = 60;
	int prevJumpingFrame = 0;
	
	// make code understand that this object is the player by setting it's flag8 on
	set(tempChar.myChar.Character, FLAG8 | SHADOW);
	
	//setup gliding and collision values	
//	c_setminmax(tempChar.myChar.Character);
	
	//setup push value to ignore lower objects
	tempChar.myChar.Character.push = -1;
	tempChar.myChar.Character.group = -1;
	
	// run function that gets combo input for player
	getKeyInputCombo(tempChar);
	
	// turn on gravity that pulls entity to the ground
	handleGrav(tempChar.myChar.Character, 8, tempChar.myChar.gravityOffset);
	
	// make sure some health is setup
	if(tempChar.myChar.Character.skill2 == 0) { tempChar.myChar.Character.skill2 = 100; tempChar.myChar.health = tempChar.myChar.Character.skill2; }
	
	// run this function as long as the entity still exists and is not dead...
	while (tempChar.myChar.health > 0) {
		// set proper health
		tempChar.myChar.health = tempChar.myChar.Character.skill2;
		// reset blocking
		reset(tempChar.myChar.Character, FLAG1);
		
		// return rotation updates etc
		getInput(tempChar);
		
		if(tempChar.myChar.movementMode == fighting) {
			// break fighting off completely and clean it up when player decides to move again! (only when not jumping!)
			if(tempChar.pressedMoveKey && prevJumpingFrame == 0) {
				// no attack; move again
				tempChar.myChar.movementMode = moving;
				currentComboRead = 0;
				tempChar.comboCount = 0;
				
				// reset player keyInput so combo's arent done several times even though different input was given
				resetKeyInputCombo(tempChar);
			}
			// ----------------------------------------
			// Do checks to see if need to do damage
			// ----------------------------------------
			if((tempChar.myChar.anim.animate < 50)  && ((tempChar.myChar.anim.animblend != blend) && (tempChar.myChar.anim.currentblend != stand))) {
				nextDamageCheck = 1; // set next attack availability
			}
			if((nextDamageCheck == 1) && (tempChar.myChar.anim.animate > 50)) {
				// reset attack check
				nextDamageCheck = 0;
				
				// trace forward from player position, based on rotation
				VECTOR traceTo;
				traceTo.x = tempChar.myChar.Character.x + cos(tempChar.myChar.Character.pan)*cos(tempChar.myChar.Character.tilt)*attackTraceDist;
				traceTo.y = tempChar.myChar.Character.y + sin(tempChar.myChar.Character.pan)*cos(tempChar.myChar.Character.tilt)*attackTraceDist;
				traceTo.z = tempChar.myChar.Character.z + sin(tempChar.myChar.Character.tilt)*attackTraceDist;
				
				var traceResult = c_trace(tempChar.myChar.Character.x, traceTo, IGNORE_ME | IGNORE_PASSABLE | USE_BOX); // do the trace
				
				// if anything was hit and that anything is the enemy;
				if(traceResult != 0) {
					if(is(you, FLAG7)) {
						// check which attack it's currently doing
						if(tempChar.myChar.anim.currentframe == punch) {
							you.skill2 -= 5;
							// create punching effect on the enemy
							createNewSnd("Punch", 10);
							createNewSnd("Hit", 10);
							ent_create("damage_enemy.tga", you.x, impactHit);
						}
						if(tempChar.myChar.anim.currentframe == doublepunch) {
							you.skill2 -= 10;
							// create punching effect on the enemy
							createNewSnd("Punch", 10);
							createNewSnd("Hit", 10);
							ent_create("damage_enemy.tga", you.x, impactHit);
						}
						if(tempChar.myChar.anim.currentframe == kick) {
							you.skill2 -= 5;
							// create punching effect on the enemy
							createNewSnd("Kick", 9);
							createNewSnd("Hit", 10);
							ent_create("damage_enemy.tga", you.x, impactHit);
						}
						if(tempChar.myChar.anim.currentframe == doublekick) {
							you.skill2 -= 10;
							// create punching effect on the enemy
							createNewSnd("Kick", 9);
							createNewSnd("Hit", 10);
							ent_create("damage_enemy.tga", you.x, impactHit);
						}
					}
				}
			}
			
			// ----------------------------------------
			// Do checks to for animating
			// ----------------------------------------
			
			// check if next attack is available for checking
			if((tempChar.myChar.anim.animate > 90) || (currentComboRead == 0) || (prevJumpingFrame == 1 && !is(tempChar.myChar.Character, FLAG3))) { // add check against jumping here!!
				nextAttackCheck = 1; // set next attack availability
			}
			
			// do a special check for jump kick
			if((prevJumpingFrame == 1) && tempChar.attackInput[currentComboRead] == key_for_str(tempChar.keySetup[5])) {
				// always do a kick
				if((tempChar.myChar.anim.currentblend != jumpkick)){
					prevComboRead = jumpkick;
					tempChar.myChar.anim.blendframe = jumpkick; 
					tempChar.myChar.anim.currentblend = jumpkick;
				}
				
				currentComboRead += 1;
			} else if(tempChar.myChar.anim.currentblend == jumpkick) { // if he was already jumpkicking, make him rotate ;)
				// make player spin kick!!
				c_rotate(tempChar.myChar.Character, vector(85 * time_step, 0, 0), IGNORE_FLAG2 | IGNORE_ME | IGNORE_PASSABLE | IGNORE_PUSH); // make him rotate FAST! :D
				
				// do damage checkas long as he's flying around
				// trace forward from player position, based on rotation
				VECTOR traceTo;
				traceTo.x = tempChar.myChar.Character.x + cos(tempChar.myChar.Character.pan)*cos(tempChar.myChar.Character.tilt)*attackTraceDist;
				traceTo.y = tempChar.myChar.Character.y + sin(tempChar.myChar.Character.pan)*cos(tempChar.myChar.Character.tilt)*attackTraceDist;
				traceTo.z = tempChar.myChar.Character.z + sin(tempChar.myChar.Character.tilt)*attackTraceDist;
				
				var traceResult = c_trace(tempChar.myChar.Character.x, traceTo, IGNORE_ME | USE_POLYGON); // do the trace
				if(traceResult != 0) {
					if(is(you, FLAG7)) {
						you.skill2 -= 25;
						// create punching effect on the enemy
						ent_create("damage_enemy.tga", you.x, impactHit);
					}
				}
			}
			
			// update only if animation is finished or when first combo is started
			if((nextAttackCheck == 1) && ((tempChar.myChar.anim.animate < 90) && (tempChar.myChar.anim.animblend != blend))) {
				// reset attack check
				nextAttackCheck = 0;
				
				// branch off in 3 possible attacks...
				// punch
				if(tempChar.attackInput[currentComboRead] == key_for_str(tempChar.keySetup[4])) {
					// increase combo data read
					currentComboRead += 1;
					
					// always do a punch
					if((tempChar.myChar.anim.currentblend != punch)){
						prevComboRead = punch;
						tempChar.myChar.anim.blendframe = punch; 
						tempChar.myChar.anim.currentblend = punch;
					}
					
					// when previousframe was punch already, do double punch!
					if((tempChar.myChar.anim.currentblend != doublepunch) && (tempChar.myChar.anim.previousframe == punch)){
						prevComboRead = doublepunch;
						tempChar.myChar.anim.blendframe = doublepunch; 
						tempChar.myChar.anim.currentblend = doublepunch; 
					}
					
					// update previousframe here
					tempChar.myChar.anim.previousframe = prevComboRead;
				} else if(tempChar.attackInput[currentComboRead] == key_for_str(tempChar.keySetup[5])) { // kick
					// increase combo data read
					currentComboRead += 1;
					
					// always do a kick
					if((tempChar.myChar.anim.currentblend != kick)){
						prevComboRead = kick;
						tempChar.myChar.anim.blendframe = kick; 
						tempChar.myChar.anim.currentblend = kick;
					}
					
					// when previousframe was punch already, do double punch!
					if((tempChar.myChar.anim.currentblend != doublekick) && (tempChar.myChar.anim.previousframe == kick)){
						prevComboRead = doublekick;
						tempChar.myChar.anim.blendframe = doublekick; 
						tempChar.myChar.anim.currentblend = doublekick; 
					}
					
					// update previousframe here
					tempChar.myChar.anim.previousframe = prevComboRead;
				} else if(tempChar.attackInput[currentComboRead] == key_for_str(tempChar.keySetup[6])) { // jump
					// set jumping
					set(tempChar.myChar.Character, FLAG3);
					// increase combo data read
					currentComboRead += 1;
					
					// always do a kick
					if((tempChar.myChar.anim.currentblend != jump)){
						prevComboRead = jump;
						tempChar.myChar.anim.blendframe = jump; 
						tempChar.myChar.anim.currentblend = jump;
					}
					
					// update previousframe here
					tempChar.myChar.anim.previousframe = prevComboRead;
				} else {
					// if no combo was found
					// no attack; move again
					tempChar.myChar.movementMode = moving;
					currentComboRead = 0;
					tempChar.comboCount = 0;
					
					// reset player keyInput so combo's arent done several times even though different input was given
					resetKeyInputCombo(tempChar);
				}
			}
		}
		
		if(tempChar.myChar.movementMode == moving || prevJumpingFrame == 1) {
			vec_set(tempChar.playerRot, camera.pan); // get camera rotation
			
			if(tempChar.pressedMoveKey) {
				// if player doesnt do the jumpkick...
				if(tempChar.myChar.anim.currentblend != jumpkick) {
					vec_set(tempCamRot, tempChar.myChar.Character.pan); // store current rotation
					vec_add(tempChar.playerRot, vector(tempChar.curHeading, 0, 0)); // add keyheading press to the player rotation
					
					// update rotations..
					ang_lerp(tempChar.myChar.Character.pan, tempChar.myChar.Character.pan, tempChar.playerRot, 0.95 * time_step); // smoothly update rotation
					tempChar.myChar.Character.tilt = tempCamRot.y; tempChar.myChar.Character.roll = tempCamRot.z; // kill tilt and roll rotation
					
					//make the player move around
					c_move(tempChar.myChar.Character, vector(10*time_step, 0, 0), nullvector, GLIDE | IGNORE_PASSABLE | IGNORE_PUSH | ACTIVATE_PUSH); // Move when key is pressed
				}
				
				// reset blending animation
				if(tempChar.myChar.anim.currentblend != move && prevJumpingFrame == 0){ tempChar.myChar.anim.blendframe = move; tempChar.myChar.anim.currentblend = move; }
			} else {
				//reset blending animation
				if(tempChar.myChar.anim.currentblend != stand && prevJumpingFrame == 0){ tempChar.myChar.anim.blendframe = stand; tempChar.myChar.anim.currentblend = stand; }
			}
		} else if(tempChar.myChar.movementMode == blocking) {
			// set animations to blend to blocking
			if(tempChar.myChar.anim.currentblend != block){ 
				tempChar.myChar.anim.blendframe = block; 
				tempChar.myChar.anim.currentblend = block; 
				
				// reset player keyInput so combo's arent done several times even though different input was given
				currentComboRead = 0;
				tempChar.comboCount = 0;
				
				resetKeyInputCombo(tempChar);
			}
			
			// set actual blocking!
			set(tempChar.myChar.Character, FLAG1); // set to be blocking
			
			tempChar.myChar.movementMode = moving; // reset character to move, if he would still be blocking, the attack loop will set this back
		}
		
		// do a check for the player, if he's still jumping...
		if(is(tempChar.myChar.Character, FLAG3)) {
				prevJumpingFrame = 1;
			} else {
				prevJumpingFrame = 0;
		}
		
		wait(1);
	}
	
	// play deathscream
	createNewSnd("Death", 5);
	
	// turn this model into a ragdoll!
	RagDoll_Spawn(tempChar.myChar.Character);
	
	if(gPlayerList.lifes[tempChar.playerID] > 0) {
		while(!key_pressed(key_for_str(tempChar.keySetup[6]))) { wait(1); } // while jump key is not being pressed
		while(key_pressed(key_for_str(tempChar.keySetup[6]))) { wait(1); } // wait untill release
		
		// respawn!
		if(mouse_mode == 0) {
			tempChar.playerReset = 1;
		}
	} else {
		createMainMenu();
	}
	
	// remove (free) when object is useless, is done upon level switch due to sys_nxalloc
//	ent_remove(tempChar.myChar.Character);
//	free(tempChar);
}

void checkPlayerRespawn() {
	var i;
	
	while(gPlayerList.playerArray[0] == NULL) { wait(1); } // first wat with the check untill any player is spawned
	
	// check as long as a player exists
	while(gPlayerList.playerArray[0] != NULL) {
		for(i=0; i<4; i++) { // check against all players
			if(gPlayerList.playerArray[i] != NULL) { // current check is not NULL
				if(gPlayerList.playerArray[i].playerReset == 1) {
					// when player needs to be reset...
					STRING* fileName = str_create("");
					str_for_entfile(fileName, gPlayerList.playerArray[i].myChar.Character);
					
					VECTOR spawnPos;
					vec_set(spawnPos.x, gPlayerList.playerArray[i].myChar.Character.x);
					
					my = ent_create(fileName, spawnPos, NULL);
					my.skill1 = gPlayerList.playerArray[i].myChar.Character.skill1;
					my.skill2 = 100;
					my.skill3 = gPlayerList.playerArray[i].myChar.Character.skill3;
					my.skill4 = gPlayerList.playerArray[i].myChar.Character.skill4;
					
					PLAYER* newPlayer = initmyPlayer();
					
					gPlayerList.lifes[i] -= 1;
					gPlayerList.playerArray[i].playerReset = 0;
				}
			}
		}
		wait(1);
	}
	
	// end this function because level is being changed
}